/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.testing.script;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.script.Compilable;
import javax.script.CompiledScript;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

import org.apache.commons.io.IOUtils;
import org.junit.Before;
import org.junit.Test;

public abstract class AbstractScriptTest {

	private String engineName = "js";

	protected String getEngineName() {
		return engineName;
	}

	protected void setEngineName(String engineName) {
		this.engineName = engineName;
	}

	protected List<String> getScriptResourceNames() throws IOException {
		final String scriptsResourceName = getScriptsResourceName();

		InputStream is = null;
		try {
			is = getClass().getResourceAsStream(scriptsResourceName);
			if (is == null) {
				throw new IOException("Could not load scripts resource ["
						+ scriptsResourceName + "].");
			}
			@SuppressWarnings("unchecked")
			final List<String> scriptResourceNames = IOUtils.readLines(is);
			return scriptResourceNames;
		} finally {
			IOUtils.closeQuietly(is);
		}

	}

	protected String getScriptsResourceName() {
		final String scriptsResourceName = "/"
				+ getClass().getName().replace('.', '/') + ".scripts";
		return scriptsResourceName;
	}

	private ScriptEngine scriptEngine;

	private List<CompiledScript> compiledScripts;

	@Before
	public void prepare() throws ScriptException, IOException {
		this.scriptEngine = getScriptEngine();
		if (!(scriptEngine instanceof Compilable)) {
			throw new ScriptException(
					"Script engine does not support compilation.");
		}
		this.compiledScripts = compileScripts((Compilable) getScriptEngine(),
				getScriptResourceNames());
	}

	protected Map<String, Object> getImplicitObjects() {
		return Collections.emptyMap();
	}

	@Test
	public void run() throws ScriptException {
		try {

			final Map<String, Object> implicitObjects = getImplicitObjects();
			for (Entry<String, Object> entry : implicitObjects.entrySet()) {
				scriptEngine.put(entry.getKey(), entry.getValue());
			}

			for (CompiledScript compiledScript : this.compiledScripts) {
				compiledScript.eval(scriptEngine.getContext());
			}
		} catch (ScriptException sex) {
			// TODO better error handling
			throw sex;
		}
	}

	protected List<CompiledScript> compileScripts(final Compilable compiler,
			final List<String> scriptResourceNames) throws ScriptException,
			IOException {
		final List<CompiledScript> compiledScripts = new LinkedList<CompiledScript>();
		for (String scriptResourceName : scriptResourceNames) {
			InputStream is = null;
			try {
				is = getClass().getResourceAsStream(scriptResourceName);
				if (is == null) {
					throw new IOException("Could not load script resource ["
							+ scriptResourceName + "].");
				}
				final CompiledScript compiledScript = compiler
						.compile(new InputStreamReader(is, "UTF-8"));
				compiledScripts.add(compiledScript);
			} finally {
				IOUtils.closeQuietly(is);
			}
		}
		return compiledScripts;
	}

	protected ScriptEngine getScriptEngine() {
		final ScriptEngineManager scriptEngineManager = getScriptEngineManager();
		final ScriptEngine scriptEngine = scriptEngineManager
				.getEngineByName(getEngineName());
		return scriptEngine;
	}

	protected ScriptEngineManager getScriptEngineManager() {
		return new ScriptEngineManager();
	}
}
