/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.testing.server.jetty;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;

import net.disy.legato.testing.server.IWebServerEnvironment;
import net.disy.legato.testing.server.IWebServerEnvironmentFactory;
import net.disy.legato.testing.server.PortFinder;
import net.disy.legato.testing.util.PropertyResolver;

import org.apache.commons.io.IOUtils;

public class PropertiesJettyWebServerEnvironmentFactory implements
		IWebServerEnvironmentFactory {

	// private static final Log logger = LogFactory
	// .getLog(PropertiesJettyWebServerEnvironmentFactory.class);

	public static final String WEBAPP_HOST_PROPERTY_NAME = "webapp.host";
	public static final String DEFAULT_HOST = "localhost";
	public static final String WEBAPP_PORT_PROPERTY_NAME = "webapp.port";
	public static final String WEBAPP_PORT_PROPERTY_VALUE_AUTO = "auto"; //$NON-NLS-1$
	public static final int DEFAULT_PORT = 8080;
	public static final String WEBAPP_CONTEXT_PROPERTY_NAME = "webapp.context";
	public static final String DEFAULT_CONTEXT = "test";
	public static final String WEBAPP_HOME_PROPERTY_NAME = "webapp.home";
	public static final String DEFAULT_HOME = ".";

	@SuppressWarnings("unused")
	private final String host;
	private final int port;
	private final String context;
	private final String home;

	public PropertiesJettyWebServerEnvironmentFactory(
			final Map<String, String> properties) {

		final String host = properties.get(WEBAPP_HOST_PROPERTY_NAME);
		final String port = properties.get(WEBAPP_PORT_PROPERTY_NAME);
		final String context = properties.get(WEBAPP_CONTEXT_PROPERTY_NAME);
		final String home = properties.get(WEBAPP_HOME_PROPERTY_NAME);

		this.host = host == null ? DEFAULT_HOST : host;
		this.context = context == null ? DEFAULT_CONTEXT : context;
		this.home = home == null ? DEFAULT_HOME : home;
		// this.port = port == null ? DEFAULT_PORT : Integer.parseInt(port);

		if (port == null) {
			this.port = DEFAULT_PORT;
		} else if (WEBAPP_PORT_PROPERTY_VALUE_AUTO.equalsIgnoreCase(port)) {
			this.port = PortFinder.findFreePort(host);
		} else {
			this.port = Integer.parseInt(port);
		}
	}

	public IWebServerEnvironment createWebServerEnvironment() {

		final JettyWebServerEnvironment webServerEnvironment = new JettyWebServerEnvironment(
				port);

		webServerEnvironment.addWebAppContext("/" + context, home);
		return webServerEnvironment;
	}

	public static PropertiesJettyWebServerEnvironmentFactory getInstance(
			final String propertiesFileName) throws IOException {
		return getInstance(new File("."), propertiesFileName);

	}

	public static PropertiesJettyWebServerEnvironmentFactory getInstance(
			final File baseDir, final String propertiesFileName)
			throws IOException {
		final Map<String, String> defaultProperties = new HashMap<String, String>();

		defaultProperties.put("baseDir", baseDir.getCanonicalFile()
				.getAbsolutePath().replace('\\', '/'));

		final Map<String, String> properties = new HashMap<String, String>();

		final File file = new File(baseDir, propertiesFileName);
		{

			final Properties draftProperties = new Properties();
			InputStream is = null;
			try {
				is = new FileInputStream(file);
				draftProperties.load(is);
			} finally {
				IOUtils.closeQuietly(is);
			}

			for (final Entry<Object, Object> entry : draftProperties.entrySet()) {
				final Object key = entry.getKey();
				final Object value = entry.getValue();
				properties.put((key == null ? null : key.toString()),
						(value == null ? null : value.toString()));
			}
		}

		final Map<String, String> resolvedProperties = new PropertyResolver(
				defaultProperties).resolve(properties);

		for (final Entry<String, String> entry : resolvedProperties.entrySet()) {
			System.setProperty(entry.getKey(), entry.getValue());
		}

		final PropertiesJettyWebServerEnvironmentFactory factory = new PropertiesJettyWebServerEnvironmentFactory(
				resolvedProperties);

		return factory;

	}

}
